# ---------------------------------------------------------------------
# Copyright (c) 2017 Kim Peter Wabersich, email: wabersich@kimpeter.de
# Permission is hereby granted, free of charge, to any person obtaining
# a copy of this software and associated documentation files (the "Software"),
# to deal in the Software without restriction, including without limitation
# the rights to use, copy, modify, merge, publish, distribute, sublicense,
# and/or sell copies of the Software, and to permit persons to whom the
# Software is furnished to do so, subject to the following conditions:
#
# The above copyright notice and this permission notice shall be included
# in all copies or substantial portions of the Software.
#
# THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND,
# EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF
# MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT.
# IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM,
# DAMAGES OR OTHER LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR
# OTHERWISE, ARISING FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER
# DEALINGS IN THE SOFTWARE.
# -----------------------------------------------------------------
# interface to robust-graph-linear-empc-solver
class InterfaceRobustGesReq(object):
    # constructor of child class
    def __init__(self):
        # planning horizon
        self.planning_horizon_n_ = 0

        # number of inputs to be applied to the real system
        self.P_ = 0

        # number of continuous states
        self.num_state_variables_real_ = 0

        # number of continuous inputs
        self.num_input_variables_real_ = 0

        # number of discrete values that graph input can attain
        self.num_graph_input_values_ = 0

        # initial state conditions
        self.x0_real_ = []
        self.x0_graph_ = []

        # state box constraints lb<= x_real <= ub (per time stamp)
        self.x_lb_ = []
        self.x_ub_ = []

        # input box constraints lb<= u_real <= ub (per time stamp)
        self.u_lb_ = []
        self.u_ub_ = []

        # graph dynamics
        self.function_graph_dynamics_ = 0

        # graph (black-box) constraint
        self.function_check_graph_constraints_ = 0

        # dynamics of the model: x_real(k+1) =
        # A(x_graph)x_real(k) + b(x_graph)u_real(k) + w^*(k)
        self.function_matrix_a_dynamics_ = 0
        self.function_matrix_b_dynamics_ = 0
        self.function_disturbance_ = []
        self.function_disturbance_nominal_ = []

        # controller for constructing the tube (P-step system)
        # u = phi(tilde v_real, tilde x_real, tilde z_real,
        #       tilde v_graph, tilde x_graph, tilde z_graph)
        self.function_tilde_phi_ = 0
        self.function_phi_ = 0

        # tightened state box constraints (nominal states)
        self.z_lb_tight_ = []
        self.z_ub_tight_ = []

        # tightened input box constraints (nominal inputs)
        self.v_lb_tight_ = []
        self.v_ub_tight_ = []

        # tightened graph constraints
        self.function_check_tilde_graph_constraints_tight_ = 0

        # the nominal stage cost function is defined via a slack variable
        # in order to increase robustness in case of piecewise linear cost
        # functions:
        # objective = min t, s.t. t>=arr_l_i(x,u) for all i...num_stage_cost_funcs_
        self.num_stage_cost_funcs_ = 0
        self.function_arr_stage_cost_ = 0
        self.function_arr_stage_cost_non_casadi_ = 0

        # black box functions for the integrated cost. defined as above.
        # args (v_real, z_real, v_graph, z_graph, RPI_set)
        self.num_int_stage_cost_funcs_ = 0
        self.function_arr_int_stage_cost_arr_ = 0
        self.function_arr_int_stage_cost_arr_non_casadi_ = 0

        # RPI set as hypercube
        self.RPI_set_lb_ = []
        self.RPI_set_ub_ = []

        # definition of the error set W (hypercube)
        self.W_set_lb = []
        self.W_set_ub = []

        # definition of the terminal region constraint function
        self.function_arr_X_f_constraint_ = 0

        # definition of the terminal cost
        self.function_V_f_ = 0

        # optimal orbit cost w.r.t. real variable values
        self.orbit_cost_opt_ = 0

        # optimal real values state orbit
        self.x_orbit_opt_ = []

        # optimal real values input orbit
        self.u_orbit_opt_ = []

        # lambdas for dissipativity
        self.lambda_ = []


# result
class InterfaceRobustGesRes(object):
    def __init__(self):
        # trajectory (graph and real states)
        self.x_real_ = []
        self.x_graph_ = []
        self.u_real_ = []
        self.u_graph_ = []

        # objective
        self.objective_ = 0

        # uniqueness w.r.t. to graph todo
        self.unique_ = False