# ---------------------------------------------------------------------
# Copyright (c) 2017 Kim Peter Wabersich, email: wabersich@kimpeter.de
# Permission is hereby granted, free of charge, to any person obtaining
# a copy of this software and associated documentation files (the "Software"),
# to deal in the Software without restriction, including without limitation
# the rights to use, copy, modify, merge, publish, distribute, sublicense,
# and/or sell copies of the Software, and to permit persons to whom the
# Software is furnished to do so, subject to the following conditions:
#
# The above copyright notice and this permission notice shall be included
# in all copies or substantial portions of the Software.
#
# THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND,
# EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF
# MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT.
# IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM,
# DAMAGES OR OTHER LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR
# OTHERWISE, ARISING FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER
# DEALINGS IN THE SOFTWARE.
# -----------------------------------------------------------------
# multidimensional integer grid counting
class IntegerGrid(object):
    # constructor and member declaration
    def __init__(self, dimension, lower_bounds, upper_bounds):
        # assign bounds
        self.upper_bounds_ = list(upper_bounds)
        self.lower_bounds_ = list(lower_bounds)

        # assign dimension
        self.dimension_ = dimension

        # init state
        self.x_ = list(lower_bounds)

    # "count" on multidimensional grid
    def count(self):
        self.x_, end_reached = self.count_rec(self.x_, 1)
        return self.x_, end_reached

    # recursive count function
    def count_rec(self, x_count, digit):
        overflow = 0
        if not (digit == self.dimension_):  # not last digit?
            (x_count, pre_overflow) = self.count_rec(x_count, digit + 1)
        else:  # last digit
            pre_overflow = 1

        # if there is an previous overflow (either because last digit or
        # previous overflow do counting
        if pre_overflow == 1:
            x_count[digit-1] += 1  # count
            if x_count[digit-1] > self.upper_bounds_[digit-1]:  # check for overflow
                x_count[digit-1] = self.lower_bounds_[digit-1]
                overflow = 1

        return x_count, overflow
