# ---------------------------------------------------------------------
# Copyright (c) 2017 Kim Peter Wabersich, email: wabersich@kimpeter.de
# Permission is hereby granted, free of charge, to any person obtaining
# a copy of this software and associated documentation files (the "Software"),
# to deal in the Software without restriction, including without limitation
# the rights to use, copy, modify, merge, publish, distribute, sublicense,
# and/or sell copies of the Software, and to permit persons to whom the
# Software is furnished to do so, subject to the following conditions:
#
# The above copyright notice and this permission notice shall be included
# in all copies or substantial portions of the Software.
#
# THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND,
# EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF
# MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT.
# IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM,
# DAMAGES OR OTHER LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR
# OTHERWISE, ARISING FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER
# DEALINGS IN THE SOFTWARE.
# -----------------------------------------------------------------
import matplotlib.pyplot as plt
import numpy as np


# plotter for simple supply chain network
class plotter_supply_chain(object):
    def plot(self, x_real, x_graph, u_real, u_graph, planning_horizon_n, plot_type,
                                            close_plot_first=True, block_plot=False):
        if close_plot_first == True:
            plt.close()
        scatter_size = 50
        fig = plt.figure(1, figsize=(20, 8))
        if plot_type == "real":
            id1 = 422
            id2 = 424
            id3 = 426
            id4 = 428
            time_x = np.array(range(0, planning_horizon_n + 1))
        if plot_type == "optimization":
            id1 = 421
            id2 = 423
            id3 = 425
            id4 = 427
            time_x = np.array(range(0, planning_horizon_n + 1))
        ax1 = fig.add_subplot(id1)
        time_u = np.array(range(0, planning_horizon_n ))
        plt.scatter(time_x, x_real[0, :], label='supplier production', color="g", alpha=0.5,
                    s=scatter_size)
        plt.scatter(time_x, x_real[1, 0:time_x.shape[0]], label='supplier storage', color="g", marker="s", alpha=0.5,
                    s=scatter_size)
        plt.scatter(time_x, x_real[2, 0:time_x.shape[0]], label='truck storage', color="b", marker="+", alpha=0.8,
                    s=scatter_size)
        plt.scatter(time_x, x_real[3, 0:time_x.shape[0]], label='retailer 1', color="r", alpha=0.5, s=scatter_size)
        plt.scatter(time_x, x_real[4, 0:time_x.shape[0]], label='retailer 2', color="b", alpha=0.5, s=scatter_size)
        plt.scatter(time_x, x_real[5, 0:time_x.shape[0]], label='retailer 3', color="m", alpha=0.5, s=scatter_size)
        #plt.xlabel("time [s]")
        #plt.ylabel("states [units]")
        if plot_type == "optimization":
            plt.legend()
        plt.grid()
        plt.title("Real state variables")
        ax1.set_xlim([-.5, planning_horizon_n + 5])

        ax2 = fig.add_subplot(id3)
        plt.scatter(time_x, x_graph[0:time_x.shape[0]], label='truck position', color="c", marker="*", alpha=0.5,
                    s=scatter_size)
        #plt.xlabel("time [s]")
        #plt.ylabel("states [units]")
        if plot_type == "optimization":
            plt.legend()
        plt.grid()
        plt.title("Graph state variables")
        ax2.set_xlim([-.5, planning_horizon_n + 5])

        ax3 = fig.add_subplot(id2)
        time = np.array(range(0, planning_horizon_n))
        plt.scatter(time_u, u_real[0, 0:time_u.shape[0]], label='truck loading', color="b", marker="+", alpha=0.8,
                    s=scatter_size)
        plt.scatter(time_u, u_real[1, 0:time_u.shape[0]], label='supplier production', color="g", alpha=0.5,
                    s=scatter_size)
        if plot_type == "optimization":
            plt.legend()
        plt.grid()
        plt.title("Real input variables")
        ax3.set_xlim([-.5, planning_horizon_n + 5])

        ax4 = fig.add_subplot(id4)
        time = np.array(range(0, planning_horizon_n))
        plt.scatter(time_u, u_graph[0:time_u.shape[0]], label='truck navigation', color="b", marker="+", alpha=0.8,
                    s=scatter_size)
        if plot_type == "optimization":
            plt.legend()
        plt.grid()
        plt.title("Graph input variables")
        ax4.set_xlim([-.5, planning_horizon_n + 5])
        plt.show(block=block_plot)
