# ---------------------------------------------------------------------
# Copyright (c) 2017 Kim Peter Wabersich, email: wabersich@kimpeter.de
# Permission is hereby granted, free of charge, to any person obtaining
# a copy of this software and associated documentation files (the "Software"),
# to deal in the Software without restriction, including without limitation
# the rights to use, copy, modify, merge, publish, distribute, sublicense,
# and/or sell copies of the Software, and to permit persons to whom the
# Software is furnished to do so, subject to the following conditions:
#
# The above copyright notice and this permission notice shall be included
# in all copies or substantial portions of the Software.
#
# THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND,
# EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF
# MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT.
# IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM,
# DAMAGES OR OTHER LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR
# OTHERWISE, ARISING FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER
# DEALINGS IN THE SOFTWARE.
# -----------------------------------------------------------------
from empc_closed_loop.simulation import empc_closed_loop_simulation
from graph_robust_empc_solver.solver import GraphRobustEmpcSolver
from supply_chain_model.configurations.with_terminal_constraints.model import get_supply_chain_robust_model
from supply_chain_model.plot import plotter_supply_chain
from supply_chain_model.simulation import SimulationSupplyChainModel
import numpy as np

# create robust simply supply chain object
ges_problem_robust = get_supply_chain_robust_model(robust='none')

# define initial condition
ges_problem_robust.x0_real_ = np.array([0.0, 0.0, 0.0, -22.0, 20.0, -10.0]) # nominal simulations
ges_problem_robust.x0_graph_ = np.array(3)

# create simulation object
simulation = SimulationSupplyChainModel(ges_problem_robust)

# create plotter object
plotter = plotter_supply_chain()

# create solver object
graph_robust_empc_solver = GraphRobustEmpcSolver(ges_problem_robust, robust=False)

# run closed loop simulation
empc_closed_loop_simulation(ges_problem_robust, simulation, plotter, graph_robust_empc_solver, real_disturbance=True,
                            save_path = 'results/run_with_terminal_constraints_closed_loop_II/')
